/*************************************************************************

    File: lambdaInt.pl
    Copyright (C) 2004,2005,2006 Patrick Blackburn & Johan Bos

    This file is part of BB1, version 1.3 (November 2006).

    Modified by Frank Richter in May 2008 to include parse tree output and
    parsing of arbitrary constituents.

    BB1 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    BB1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BB1; if not, write to the Free Software Foundation, Inc., 
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*************************************************************************/


:- module(lambdaInt,[info/0,
                     infix/0,
		     interface/0,
		     pp_cat/2,
		     part_parse/1,
                     prefix/0,
                     lambda/0,
                     lambda/2,
                     lambdaTestSuite/0]).

:- use_module(library(system), [exec/3]).

:- use_module(readLine,[readLine/1]).

:- use_module(comsemPredicates,[appendLists/3,
				infix/0,
                                prefix/0,
                                printRepresentations/1,
				compose/3]).

:- use_module(betaConversion,[betaConvert/2]).

:- use_module(sentenceTestSuite,[sentence/2]).

:- [englishGrammarInt].

:- [partialParses].

:- [englishLexicon].

:- [semLexLambda].

:- [semRulesLambda].


/*========================================================================
   Blackburn and Bos' Driver Predicates
========================================================================*/

lambda:-
	readLine(Sentence),
        lambda(Sentence,Sems),
	printRepresentations(Sems).

lambda(Sentence,Sems):-
	setof(Sem,t(_,[sem:Sem],Sentence,[]),Sems).

/*========================================================================
   Driver Predicate for Graphical Interface (with Auxiliary Predicates)
========================================================================*/

interface:-
	exec('java -jar tv.jar', [pipe(StdOut), std, std], _Pid),
        repeat,
	   readLine(S),
           % initialize the tree counter
           retractall(tree_cnt_(_)), assert(tree_cnt_(0)),
           % retrieve all parse trees, incrementing the tree
           % counter on each
           findall(Sem,
               (t(Tree,[sem:Sem],S,[]), show_tree(StdOut,Tree), inc_tree_count),Sems),
           nl,
	   write('The sentence receives the following truth conditions:'),
	   nl,
	   printRepresentations(Sems),
	   nl,nl,
           fail.


% Increments the global tree counter. The counter is reset
% before each sentence is parsed.
%
inc_tree_count :-
    tree_cnt_(X),
    retractall(tree_cnt_(_)),
    X1 is X + 1,
    assert(tree_cnt_(X1)),
    write(X1), write(' '), !.


% Sends a tree to the tree viewer.
%
show_tree(Tree) :- show_tree(treg,Tree).
show_tree(Stream,Tree) :-
  % send the window's title
  write(Stream, Tree), nl(Stream),
  
  % send the tree
  print_tree(Stream,Tree), nl(Stream).


% print_tree/2 writes a tree represented as a Prolog term to
% the tree viewer's input stream in its format.
%

% leaf case
print_tree(Stream,Leaf) :-
    functor(Leaf,_,0), !,
    write(Stream, '("'),
    write(Stream, Leaf),
    write(Stream, '" [style=bolditalic color=blue])').

% internal node case
print_tree(Stream,Tree) :-
    functor(Tree,Root,Arr),
    write(Stream, '("'),
    write(Stream, Root),
    write(Stream, '" [style=bold]'),
    print_args(Stream,Tree,0,Arr),
    write(Stream, ')').

print_args(_,_,N,N) :- !.
print_args(Stream,Tree,I,N) :-
    I1 is I+1,
    arg(I1,Tree,Arg),
    print_tree(Stream,Arg), 
    print_args(Stream,Tree,I1,N).

/*========================================================================
   Driver Predicates for Partial Parses (with Auxiliary Predicates)
========================================================================*/

pp_cat(Cat,List) :-
	findall(Acc,
	        (partial_parse(Cat,List,Tree,Sem), Acc = coll(Tree-Sem)),
	        Accs),
	listParseResultsCheck(Accs).

part_parse(List) :-
	pp_cat(_,List).


listParseResultsCheck([]) :-
	nl,write('no successful parses'), nl, !.

listParseResultsCheck(Accs) :-
	listParseResults(Accs,0).


listParseResults([],_) :-
	write('-----------------------------------------------------------'),nl.
	
listParseResults([coll(Tree-Sem)|Rest],Counter) :-
	CounterN is Counter + 1,
	write('-----------------------------------------------------------'),nl,
	write('('), write(CounterN), write(')'), nl,
	betaConvert(Sem,ConvertedSem),
	write('Semantic Representation: '), nl,
	\+ \+ (numbervars(Sem,0,_), print(Sem)),nl,nl,
	write('After lambda conversion: '), nl,
	\+ \+ (numbervars(ConvertedSem,0,_), print(ConvertedSem)),nl,nl,
	write('Syntactic Structure: '),
	write(Tree),nl,nl,
	listParseResults(Rest,CounterN).

/*========================================================================
   Test Suite Predicates
========================================================================*/

lambdaTestSuite:-
	nl, write('>>>>> LAMBDA ON SENTENCE TEST SUITE <<<<< '), nl,
        sentence(Sentence,_),
        nl, write('Sentence: '), write(Sentence),
	lambda(Sentence,Formulas),
	printRepresentations(Formulas),
        fail.

lambdaTestSuite.


/*========================================================================
   Info
========================================================================*/

info:-
   format('~n> ------------------------------------------------------------------ <',[]),
   format('~n> lambdaInt.pl, by Patrick Blackburn and Johan Bos                   <',[]),
   format('~n>            modified by Frank Richter                               <',[]),
   format('~n>                                                                    <',[]),
   format('~n> ?- lambda.              - parse a typed-in sentence                <',[]),
   format('~n> ?- interface.           - parse a typed-in sentence and show tree  <',[]),
   format('~n> ?- pp_cat(cat,[...]).   - parse a list of strings of category cat  <',[]),
   format('~n> ?- part_parse(List)     - parse a list of strings                  <',[]),
   format('~n> ?- lambda(S,F).         - parse a sentence and return formula      <',[]),
   format('~n> ?- lambdaTestSuite.     - run the test suite                       <',[]),
   format('~n> ?- infix.               - switches to infix display mode           <',[]),
   format('~n> ?- prefix.              - switches to prefix display mode          <',[]),
   format('~n> ?- info.                - shows this information                   <',[]),
   format('~n> ------------------------------------------------------------------ <',[]),
   format('~n~n',[]).


/*========================================================================
   Display info at start
========================================================================*/

:- info.
